﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PIbd_22_Kalyshev_Y_V_MotorBoat_Base
{
    internal class DrawningBoat
    {
        /// <summary>
        /// Класс-сущность
        /// </summary>
        public EntityBoat Boat { private set; get; }
        /// <summary>
        /// Левая координата отрисовки автомобиля
        /// </summary>
        private float _startPosX;
        /// <summary>
        /// Верхняя кооридната отрисовки автомобиля
        /// </summary>
        private float _startPosY;
        /// <summary>
        /// Ширина окна отрисовки
        /// </summary>
        private int? _pictureWidth = null;
        /// <summary>
        /// Высота окна отрисовки
        /// </summary>
        private int? _pictureHeight = null;
        /// <summary>
        /// Ширина отрисовки автомобиля
        /// </summary>
        private readonly int _boatWidth = 170;
        /// <summary>
        /// Высота отрисовки автомобиля
        /// </summary>
        private readonly int _boatHeight = 60;
        /// <summary>
        /// Инициализация свойств
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес автомобиля</param>
        /// <param name="bodyColor">Цвет кузова</param>
        public void Init(int speed, float weight, Color bodyColor)
        {
            Boat = new EntityBoat();
            Boat.Init(speed, weight, bodyColor);
        }
        /// <summary>
        /// Установка позиции автомобиля
        /// </summary>
        /// <param name="x">Координата X</param>
        /// <param name="y">Координата Y</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        public void SetPosition(int x, int y, int width, int height)
        {
            if (x + _boatWidth <= width && y + _boatHeight <= height && x > 0 && y > 0)
            {
                _startPosX = x;
                _startPosY = y;
                _pictureWidth = width;
                _pictureHeight = height;
            }
        }
        /// <summary>
        /// Изменение направления перемещения
        /// </summary>
        /// <param name="direction">Направление</param>
        public void MoveTransport(Direction direction)
        {
            if (!_pictureWidth.HasValue || !_pictureHeight.HasValue)
            {
                return;
            }
            switch (direction)
            {
                // вправо
                case Direction.Right:
                    if (_startPosX + _boatWidth + Boat.Step < _pictureWidth)
                    {
                        _startPosX += Boat.Step;
                    }
                    break;
                //влево
                case Direction.Left:
                    if (_startPosX - Boat.Step > 0)
                    {
                        _startPosX -= Boat.Step;
                    }
                    break;
                //вверх
                case Direction.Up:
                   if (_startPosY - Boat.Step > 0)
                   {
                        _startPosY -= Boat.Step;
                   }
                    break;
                //вниз
                case Direction.Down:
                    if (_startPosY + _boatHeight + Boat.Step < _pictureHeight)
                    {
                        _startPosY += Boat.Step;
                    }
                    break;
            }
        }
        /// <summary>
        /// Отрисовка лодки
        /// </summary>
        /// <param name="g"></param>
        public void DrawTransport(Graphics g)
        {
            if (_startPosX < 0 || _startPosY < 0
            || !_pictureHeight.HasValue || !_pictureWidth.HasValue)
            {
                return;
            }
            Pen pen = new(Color.Black, 3);
            Brush br = new SolidBrush(Boat?.BodyColor ?? Color.Black);
            Brush brBrown = new SolidBrush(Color.Brown);
            // Внешняя часть лодки
            g.DrawLine(pen, _startPosX, _startPosY, _startPosX + 120, _startPosY + 0);
            g.DrawLine(pen, _startPosX + 120, _startPosY, _startPosX + 170, _startPosY + 30);
            g.DrawLine(pen, _startPosX + 170, _startPosY + 30, _startPosX + 120, _startPosY + 60);
            g.DrawLine(pen, _startPosX + 120, _startPosY + 60, _startPosX, _startPosY + 60);
            g.DrawLine(pen, _startPosX, _startPosY + 60, _startPosX, _startPosY);
            PointF pt1 = new PointF(_startPosX, _startPosY);
            PointF pt2 = new PointF(_startPosX + 120, _startPosY);
            PointF pt3 = new PointF(_startPosX + 170, _startPosY + 30);
            PointF pt4 = new PointF(_startPosX + 120, _startPosY + 60);
            PointF pt5 = new PointF(_startPosX, _startPosY + 60);
            g.FillPolygon(br, new PointF[] { pt1, pt2, pt3, pt4, pt5});
            // Внутренняя часть лодки
            g.FillEllipse(brBrown, _startPosX + 10, _startPosY + 10, 110, 40);
        }
        /// <summary>
        /// Смена границ формы отрисовки
        /// </summary>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        public void ChangeBorders(int width, int height)
        {
            _pictureWidth = width;
            _pictureHeight = height;
            if (_pictureWidth <= _boatWidth || _pictureHeight <= _boatHeight)
            {
                _pictureWidth = null;
                _pictureHeight = null;
                return;
            }
            if (_startPosX + _boatWidth > _pictureWidth)
            {
                _startPosX = _pictureWidth.Value - _boatWidth;
            }
            if (_startPosY + _boatHeight > _pictureHeight)
            {
                _startPosY = _pictureHeight.Value - _boatHeight;
            }
        }
    }
}
